unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    RangeRadios: TRadioGroup;
    DACRadios: TRadioGroup;
    DACOutGroup: TGroupBox;
    MinButton: TButton;
    MidButton: TButton;
    MaxButton: TButton;
    GroupBox1: TGroupBox;
    LoLabel: TLabel;
    LoTrack: TTrackBar;
    GroupBox2: TGroupBox;
    HiLabel: TLabel;
    HiTrack: TTrackBar;
    BlurbMemo: TMemo;
    Panel1: TPanel;
    WriteOneButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure LoTrackChange(Sender: TObject);
    procedure HiTrackChange(Sender: TObject);
    procedure MaxButtonClick(Sender: TObject);
    procedure MidButtonClick(Sender: TObject);
    procedure MinButtonClick(Sender: TObject);
    procedure WriteOneButtonClick(Sender: TObject);
    procedure WriteAllButtonClick(Sender: TObject);
    procedure RangeRadiosClick(Sender: TObject);
    procedure DACRadiosClick(Sender: TObject);
  protected
    procedure BuildCalFile;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOUSB, DetectUnit, WaitUnit;

const
  NybbleToHex: array[0..15] of Char = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F');

const
  //Currently in quarter-seconds.
  EstEraseTime = 2;
  Est1To1Time  = 17;
  EstWriteTime = 54;

var
  DeviceIndex: LongWord;
  Channels: Integer;
  CalFN: String;

procedure DumpStringToFile(S: AnsiString; Path: String);
var
  Fil: TFileStream;

begin
  Fil := TFileStream.Create(Path, fmCreate or fmShareDenyWrite);
  try
    Fil.Write(S[1], Length(S));
  finally
    Fil.Free;
  end;
end;

function DumpFileToString(Path: String): AnsiString;
var
  Fil: TFileStream;
  L: Integer;

begin
  Fil := TFileStream.Create(Path, fmOpenRead or fmShareDenyNone);
  try
    L := Fil.Size;
    SetString(Result, nil, L);
    Fil.Read(Result[1], L);
  finally
    Fil.Free;
  end;
end;

function GetTempDir: String;
var
  L: Integer;

begin
  SetString(Result, nil, MAX_PATH);
  L := GetTempPath(Length(Result), @Result[1]);
  SetLength(Result, L);
  Result := IncludeTrailingBackslash(Result);
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  Status, PID: LongWord;
  DetectForm: TDetectForm;

begin
  WriteOneButton.Align := alClient;

  CalFN := GetTempDir + 'Cal.txt';

  DeviceIndex := ResolveDeviceIndex(diOnly);
  Status := QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
  if (Status <> ERROR_SUCCESS) or (PID < $8070) or (PID > $807F) then begin
    DetectForm := TDetectForm.Create(Self);
    if DetectForm.ShowModal = mrOK then begin
      DeviceIndex := DetectForm.DeviceIndex;
      DetectForm.Free;
      QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
    end
    else begin
      DetectForm.Free;
      Application.Terminate;
      Exit;
    end;
  end;

  case PID of
    $8070, $8071, $8078, $8079: Channels := 16;
    $8072, $8073, $807A, $807B: Channels := 12;
    $8074, $8075, $807C, $807D: Channels := 8;
    $8076, $8077, $807E, $807F: Channels := 4;
    else Channels := 0;
  end;

  while DACRadios.Items.Count > Channels do DACRadios.Items.Delete(Channels);

  RangeRadiosClick(RangeRadios);

  BlurbMemo.Lines.Text := 'First, select the device''s range on the far left.'
            + #13#10#13#10'Select a DAC, and the calibration values from the device will be loaded in the "Low" and "High" sliders.'
            + #13#10#13#10'Use the "Out" buttons to set the DAC to its minimum, middle, or maximum value, and measure the error. Convert it to counts; 1 count is .076mV at 0-5V, .153mV at 0-10V or 5V, or .305mV at 10V.'
            + #13#10#13#10'Use the "Low" slider to adjust the low end of the calibration, and similarly the "High" slider for the high end.'
            + #13#10#13#10'Click "Write To DAC", and wait(about 15 seconds).'
            + #13#10#13#10'Try the "Out" buttons again. Repeat until the channel is calibrated.'
  ;
end;

procedure TMainForm.RangeRadiosClick(Sender: TObject);
var
  I: Integer;

begin
  case RangeRadios.ItemIndex of
    0, 2: begin
      for I := 0 to 15 do DACDirect(DeviceIndex, I, $8000);
      DACSetBoardRange(DeviceIndex, 1);
    end;
    1, 3: begin
      for I := 0 to 15 do DACDirect(DeviceIndex, I, $0000);
      DACSetBoardRange(DeviceIndex, 0);
    end;
  end;

  DACRadiosClick(DACRadios);
end;

function _FlashRead(DeviceIndex, StartAddress: LongWord; var DataSize: LongWord; pData: Pointer): LongWord;
const
  AUR_FLASH_READWRITE     = $B5;

begin
  Result := GenericVendorRead(DeviceIndex, AUR_FLASH_READWRITE, StartAddress shr 1, StartAddress shr 17, DataSize, pData);
end;

procedure TMainForm.DACRadiosClick(Sender: TObject);
var
  iDAC: Integer;
  CalData: Word;
  CalDataSize: LongWord;

begin
  DACOutGroup.Caption := 'Ch ' + IntToStr(DACRadios.ItemIndex);

  iDAC := DACRadios.ItemIndex;

  CalData := $07FE;
  CalDataSize := SizeOf(CalData);
  _FlashRead(DeviceIndex, iDAC * $20000 + $00000, CalDataSize, @CalData);
  LoTrack.Position := CalData xor $FFFF;
  LoTrackChange(LoTrack);

  CalData := $F801;
  CalDataSize := SizeOf(CalData);
  AIOUSB_FlashRead(DeviceIndex, iDAC * $20000 + $1FFFE, CalDataSize, @CalData);
  HiTrack.Position := CalData xor $FFFF;
  HiTrackChange(HiTrack);
end;

procedure TMainForm.MaxButtonClick(Sender: TObject);
begin
  DACDirect(DeviceIndex, DACRadios.ItemIndex, $FFFF);
end;

procedure TMainForm.MidButtonClick(Sender: TObject);
begin
  DACDirect(DeviceIndex, DACRadios.ItemIndex, $8000);
end;

procedure TMainForm.MinButtonClick(Sender: TObject);
begin
  DACDirect(DeviceIndex, DACRadios.ItemIndex, $0000);
end;

procedure TMainForm.LoTrackChange(Sender: TObject);
begin
  LoLabel.Caption := IntToHex(LoTrack.Position xor $FFFF, 4);
end;

procedure TMainForm.HiTrackChange(Sender: TObject);
begin
  HiLabel.Caption := IntToHex(HiTrack.Position xor $FFFF, 4);
end;

procedure TMainForm.WriteOneButtonClick(Sender: TObject);
begin
  BuildCalFile;

  WaitForm.GoWait(EstWriteTime);
  DACSetChannelCal(DeviceIndex, DACRadios.ItemIndex, PChar(CalFN));
  Sleep(1000);
  WaitForm.Hide;
end;

procedure TMainForm.WriteAllButtonClick(Sender: TObject);
var
  iChannel: Integer;

begin
  BuildCalFile;

  for iChannel := 0 to Channels-1 do begin
    WaitForm.GoWait((Channels - iChannel) * EstWriteTime);
    DACSetChannelCal(DeviceIndex, 0, PChar(CalFN));
    Sleep(1000);
  end;
  WaitForm.Hide;
end;

procedure TMainForm.BuildCalFile;
var
  I, J: Integer;
  LoCt, HiCt: Integer;
  F: Double;
  Buf: AnsiString;

begin
  LoCt := LoTrack.Position xor $FFFF;
  HiCt := HiTrack.Position xor $FFFF;

  SetLength(Buf, $70000);

  for I := 0 to $FFFF do begin
    F := (I - 0) / $FFFF;
    F := F * (HiCt - LoCt) + LoCt;
    J := Round(F);
    //Fil.WriteStr('$' + IntToHex(J, 4) + #13#10);
    Buf[I * 7 + 7] := #10;
    Buf[I * 7 + 6] := #13;
    Buf[I * 7 + 5] := NybbleToHex[J and $F];
    J := J shr 4;
    Buf[I * 7 + 4] := NybbleToHex[J and $F];
    J := J shr 4;
    Buf[I * 7 + 3] := NybbleToHex[J and $F];
    J := J shr 4;
    Buf[I * 7 + 2] := NybbleToHex[J and $F];
    Buf[I * 7 + 1] := '$';
  end;

  DumpStringToFile(Buf, CalFN);
end;

end.
